/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <cstring>

#include "app_control_uri.h"
#include "uri_internal.h"

#undef URI_API
#define URI_API extern "C" __attribute__((visibility("default")))

using namespace appcontrol;

URI_API int app_control_uri_builder_create(app_control_uri_builder_h* handle) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  Uri::Builder* builder = new (std::nothrow) Uri::Builder();
  if (builder == nullptr)
    return APP_CONTROL_URI_ERROR_OUT_OF_MEMORY;
  *handle = static_cast<app_control_uri_builder_h>(builder);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_set_scheme(app_control_uri_builder_h handle,
    const char* scheme) {
  if (handle == nullptr || scheme == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* builder = static_cast<Uri::Builder*>(handle);
  builder->SetScheme(scheme);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_set_authority(
    app_control_uri_builder_h handle, const char* auth) {
  if (handle == nullptr || auth == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* builder = static_cast<Uri::Builder*>(handle);
  builder->SetAuthority(auth);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_set_path(app_control_uri_builder_h handle,
    const char* path) {
  if (handle == nullptr || path == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* builder = static_cast<Uri::Builder*>(handle);
  builder->SetPath(path);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_add_path(app_control_uri_builder_h handle,
    const char* path) {
  if (handle == nullptr || path == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* builder = static_cast<Uri::Builder*>(handle);
  builder->AddPath(path);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_set_fragment(
    app_control_uri_builder_h handle, const char* fragment) {
  if (handle == nullptr || fragment == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* builder = static_cast<Uri::Builder*>(handle);
  builder->SetFragment(fragment);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_add_query(app_control_uri_builder_h handle,
    const char* key, const char* val) {
  if (handle == nullptr || key == nullptr || val == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* builder = static_cast<Uri::Builder*>(handle);
  builder->AddQuery(key, val);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_build(app_control_uri_builder_h builder,
    app_control_uri_h* app_control_uri) {
  if (builder == nullptr || app_control_uri == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* b = static_cast<Uri::Builder*>(builder);
  auto* u = new (std::nothrow) Uri(b->Build());
  if (u == nullptr)
    return APP_CONTROL_URI_ERROR_OUT_OF_MEMORY;
  *app_control_uri = static_cast<app_control_uri_h>(u);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_builder_destroy(app_control_uri_builder_h handle) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* builder = static_cast<Uri::Builder*>(handle);
  delete builder;
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_create(const char* encoded_app_control_uri,
    app_control_uri_h* handle) {
  if (encoded_app_control_uri == nullptr || handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* u = new (std::nothrow) Uri(encoded_app_control_uri);
  if (u == nullptr)
    return APP_CONTROL_URI_ERROR_OUT_OF_MEMORY;
  *handle = static_cast<app_control_uri_h>(u);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_destroy(app_control_uri_h handle) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  delete app_control_uri;
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_encode(app_control_uri_h handle,
    char **encoded_app_control_uri) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *encoded_app_control_uri = strdup(app_control_uri->Encode().c_str());
  if (*encoded_app_control_uri == nullptr)
    return APP_CONTROL_URI_ERROR_OUT_OF_MEMORY;

  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_scheme(app_control_uri_h handle,
    const char **scheme) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *scheme = app_control_uri->GetScheme().c_str();
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_authority(app_control_uri_h handle,
    const char **authority) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *authority = app_control_uri->GetAuthority().c_str();
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_path(app_control_uri_h handle,
    const char **path) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *path = app_control_uri->GetPath().c_str();
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_fragment(app_control_uri_h handle,
    const char **fragment) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *fragment = app_control_uri->GetFragment().c_str();
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_query(app_control_uri_h handle,
    app_control_uri_query_h* query_handle) {
  if (handle == nullptr || query_handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  const auto& ql = app_control_uri->GetQuery();
  *query_handle = static_cast<app_control_uri_query_h>(&ql);
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_host(app_control_uri_h handle,
    const char **host) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *host = app_control_uri->GetHost().c_str();
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_port(app_control_uri_h handle,
    const char **port) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *port = app_control_uri->GetPort().c_str();
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_get_user(app_control_uri_h handle,
    const char **user) {
  if (handle == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;
  auto* app_control_uri = static_cast<Uri*>(handle);
  *user = app_control_uri->GetUser().c_str();
  return APP_CONTROL_URI_ERROR_NONE;
}

URI_API int app_control_uri_query_foreach(app_control_uri_query_h handle,
    app_control_uri_query_foreach_cb callback,
    void* data) {
  if (handle == nullptr || callback == nullptr)
    return APP_CONTROL_URI_ERROR_INVALID_PARAMETER;

  const auto* ql = static_cast<const std::list<std::pair<std::string,
      std::string>>*>(handle);
  for (auto& i : *ql) {
    if (!callback(i.first.c_str(), i.second.c_str(), data))
      break;
  }
  return APP_CONTROL_URI_ERROR_NONE;
}
