/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <glib/gi18n.h>

#include <memory>

#include "uri_internal.h"

namespace appcontrol {

Uri::Builder& Uri::Builder::SetScheme(std::string scheme) {
  scheme_ = std::move(scheme);
  return *this;
}

Uri::Builder& Uri::Builder::SetAuthority(std::string auth) {
  auth_ = std::move(auth);
  return *this;
}

Uri::Builder& Uri::Builder::SetPath(std::string path) {
  path_ = std::move(path);
  return *this;
}

Uri::Builder& Uri::Builder::AddPath(std::string path) {
  if (path[0] != '/')
    path_ += "/";
  path_ += path;
  return *this;
}

Uri::Builder& Uri::Builder::SetFragment(std::string fragment) {
  fragment_ = std::move(fragment);
  return *this;
}

Uri::Builder& Uri::Builder::AddQuery(std::string key, std::string val) {
  query_list_.emplace_back(std::move(key), std::move(val));
  return *this;
}

Uri Uri::Builder::Build() {
  return Uri(std::move(scheme_), std::move(auth_), std::move(path_),
      std::move(query_list_), std::move(fragment_));
}

Uri::Uri(std::string encoded_uri) {
  std::unique_ptr<char, decltype(std::free)*> d_uri(
      g_uri_unescape_string(encoded_uri.c_str(), NULL), std::free);
  std::string uri(d_uri.get());
  Parse(uri);
}

Uri::Uri(std::string scheme, std::string auth, std::string path,
         std::list<std::pair<std::string, std::string>> query,
         std::string fragment) :
    scheme_(std::move(scheme)), auth_(std::move(auth)), path_(std::move(path)),
    fragment_(std::move(fragment)), query_list_(std::move(query)) {
  ParseAuthoritySubcomponent(auth_);
}

const std::string& Uri::GetScheme() const {
  return scheme_;
}

const std::string& Uri::GetAuthority() const {
  return auth_;
}

const std::string& Uri::GetPath() const {
  return path_;
}

const std::string& Uri::GetFragment() const {
  return fragment_;
}

const std::list<std::pair<std::string, std::string>>& Uri::GetQuery() const {
  return query_list_;
}

const std::string& Uri::GetHost() const {
  return host_;
}

const std::string& Uri::GetPort() const {
  return port_;
}

const std::string& Uri::GetUser() const {
  return user_;
}

std::string Uri::Encode() const {
  std::unique_ptr<char, decltype(std::free)*> e_scheme(
      g_uri_escape_string(scheme_.c_str(), NULL, TRUE), std::free);
  std::unique_ptr<char, decltype(std::free)*> e_auth(
      g_uri_escape_string(auth_.c_str(), NULL, TRUE), std::free);
  std::unique_ptr<char, decltype(std::free)*> e_path(
      g_uri_escape_string(path_.c_str(), NULL, TRUE), std::free);
  std::unique_ptr<char, decltype(std::free)*> e_fragment(
      g_uri_escape_string(fragment_.c_str(), NULL, TRUE), std::free);

  std::string ret = e_scheme.get();
  ret += ":";
  ret += e_auth.get();
  ret += e_path.get();
  bool first = true;

  for (auto& i : query_list_) {
    std::unique_ptr<char, decltype(std::free)*> key(
      g_uri_escape_string(i.first.c_str(), NULL, TRUE), std::free);
    std::unique_ptr<char, decltype(std::free)*> val(
      g_uri_escape_string(i.second.c_str(), NULL, TRUE), std::free);
    if (first) {
      first = false;
      ret += "?";
    } else {
      ret += "&";
    }

    ret += key.get();
    ret += "=";
    ret += val.get();
  }

  ret += "#";
  ret += e_fragment.get();

  return ret;
}

void Uri::Parse(const std::string& uri) {
  std::size_t idx = 0;

  ParseScheme(uri, idx);
  ParseAuthority(uri, idx);
  ParsePath(uri, idx);
  ParseQuery(uri, idx);
  ParseFragment(uri, idx);
}

void Uri::ParseScheme(const std::string& uri, std::size_t& idx) {
  std::size_t found = uri.find(":");
  if (found == std::string::npos)
    return;

  scheme_ = uri.substr(idx, found - idx);
  idx = found + 1;
}

void Uri::ParseAuthority(const std::string& uri, std::size_t& idx) {
  const std::string delim("//");
  std::size_t found = uri.find(delim, idx);
  if (found == std::string::npos)
    return;

  idx = found;
  found = uri.find("/", idx + delim.length());
  auth_ = uri.substr(idx, found - idx);
  idx = found;

  ParseAuthoritySubcomponent(auth_);
}

void Uri::ParsePath(const std::string& uri, std::size_t& idx) {
  std::size_t found = uri.find("?", idx);
  if (found == std::string::npos) {
    found = uri.find("#", idx);
    if (found == std::string::npos)
      found = uri.length();
  }

  path_ = uri.substr(idx, found - idx);
  idx = found;
}

void Uri::ParseQuery(const std::string& uri, std::size_t& idx) {
  if (idx == uri.length())
    return;

  std::size_t found = uri.find("?", idx);
  if (found == std::string::npos)
    return;

  idx = found;
  found = uri.find("#", idx);
  if (found == std::string::npos)
    found = uri.length();

  std::string query = uri.substr(idx, found - idx);
  idx = found;

  std::size_t i = query.find("?") + 1;
  found = query.find("&");
  while (found != std::string::npos) {
    AddQuery(query.substr(i, found - i));
    i = found + 1;
    found = query.find("&", i);
  }
  AddQuery(query.substr(i, found - i));
}

void Uri::ParseFragment(const std::string& uri, std::size_t& idx) {
  if (idx == uri.length())
    return;

  std::size_t found = uri.find("#", idx);
  if (found == std::string::npos)
    return;

  idx = found + 1;
  fragment_ = uri.substr(idx);
}

void Uri::ParseAuthoritySubcomponent(const std::string& auth) {
  if (auth.empty())
    return;

  std::size_t idx = std::string("//").length();
  std::size_t found = auth.find("@");
  if (found != std::string::npos) {
    user_ = auth.substr(idx, found - idx);
    idx = found + 1;
  }

  found = auth.find_last_of(":");
  if (found != std::string::npos && IsNumber(auth.substr(found + 1))) {
    host_ = auth.substr(idx, found - idx);
    port_ = auth.substr(found + 1);
  } else {
    host_ = auth.substr(idx);
  }
}

void Uri::AddQuery(const std::string& query) {
  std::size_t found = query.find("=");
  if (found == std::string::npos)
    return;

  query_list_.emplace_back(query.substr(0, found), query.substr(found + 1));
}

bool Uri::IsNumber(const std::string& s) {
  std::string::const_iterator iter = s.begin();
  while (iter != s.end() && std::isdigit(*iter))
    iter++;

  return (!s.empty() && iter == s.end());
}

}  // namespace appcontrol
