/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef URI_INTERNAL_H_
#define URI_INTERNAL_H_

#include <algorithm>
#include <string>
#include <list>
#include <utility>

namespace appcontrol {

class Uri {
 public:
  class Builder {
   public:
    Builder& SetScheme(std::string scheme);
    Builder& SetAuthority(std::string auth);
    Builder& SetPath(std::string path);
    Builder& AddPath(std::string path);
    Builder& SetFragment(std::string fragment);
    Builder& AddQuery(std::string key, std::string val);
    Uri Build();

   private:
    std::string scheme_;
    std::string auth_;
    std::string path_;
    std::string fragment_;
    std::list<std::pair<std::string, std::string>> query_list_;
  };

  explicit Uri(std::string encoded_uri);
  virtual ~Uri() {}

  const std::string& GetScheme() const;
  const std::string& GetAuthority() const;
  const std::string& GetPath() const;
  const std::string& GetFragment() const;
  const std::list<std::pair<std::string, std::string>>& GetQuery() const;
  const std::string& GetHost() const;
  const std::string& GetPort() const;
  const std::string& GetUser() const;
  std::string Encode() const;

 private:
  Uri(std::string scheme, std::string auth, std::string path,
      std::list<std::pair<std::string, std::string>> query,
      std::string fragment);

  bool IsNumber(const std::string& s);
  void Parse(const std::string& uri);
  void ParseScheme(const std::string& uri, std::size_t& idx);
  void ParseAuthority(const std::string& uri, std::size_t& idx);
  void ParsePath(const std::string& uri, std::size_t& idx);
  void ParseQuery(const std::string& uri, std::size_t& idx);
  void ParseFragment(const std::string& uri, std::size_t& idx);
  void ParseAuthoritySubcomponent(const std::string& auth);
  void AddQuery(const std::string& query);

 private:
  std::string scheme_;
  std::string auth_;
  std::string path_;
  std::string fragment_;
  std::list<std::pair<std::string, std::string>> query_list_;
  std::string host_;
  std::string port_;
  std::string user_;
};

}  // namespace appcontrol

#endif  // URI_INTERNAL_H_
