/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <gtest/gtest.h>
#include <app_control.h>
#include <app_control_uri.h>

#include <iostream>
#include <memory>

#include "uri_internal.h"

// The source of the following example is "https://en.wikipedia.org/wiki/Uniform_Resource_Identifier"
static const char EXAM_URI1[] = "https://john.doe@www.example.com:123";
extern int gargc;
extern char** gargv;

class AppControlTest : public testing::Test {
 public:
  virtual ~AppControlTest() {
    if (builder_ != nullptr)
      app_control_uri_builder_destroy(builder_);
    if (uri_ != nullptr)
      app_control_uri_destroy(uri_);
  }

  virtual void SetUp() {
    app_control_uri_builder_create(&builder_);
    EXPECT_NE(builder_, nullptr);

    app_control_uri_builder_set_scheme(builder_, "https");
    app_control_uri_builder_set_authority(builder_,
        "//john.doe@www.example.com:123");
    app_control_uri_builder_set_path(builder_, "/forum/questions/");
    app_control_uri_builder_set_fragment(builder_, "top");
    app_control_uri_builder_add_query(builder_, "tag", "networking");
    app_control_uri_builder_add_query(builder_, "order", "newest");

    app_control_uri_builder_build(builder_, &uri_);
    EXPECT_NE(uri_, nullptr);
  }

  virtual void TearDown() {
    count_ = 0;

    if (builder_ != nullptr) {
      auto builder =
        std::unique_ptr<void, decltype(app_control_uri_builder_destroy)*>(
          builder_, app_control_uri_builder_destroy);
      builder_ = nullptr;
    }
    if (uri_ != nullptr) {
      auto uri = std::unique_ptr<void, decltype(app_control_uri_destroy)*>(uri_,
          app_control_uri_destroy);
      uri_ = nullptr;
    }
  }

  app_control_uri_builder_h builder_;
  app_control_uri_h uri_;
  int count_ = 0;
};

/*
 * @testcase app_control_create_from_uri_handle_p
 * @description Creates a app_control handle from uri handle.
 * @apicovered app_control_create_from_uri_handle
 */
TEST_F(AppControlTest, app_control_create_from_uri_handle_p) {
  app_control_h app_control;
  char *uri;
  char *query_tag;
  int r = app_control_create_from_uri_handle(&app_control, uri_);
  EXPECT_EQ(r, APP_CONTROL_ERROR_NONE);

  r = app_control_get_uri(app_control, &uri);
  EXPECT_EQ(r, APP_CONTROL_ERROR_NONE);
  EXPECT_STREQ(uri, EXAM_URI1);

  r = app_control_get_extra_data(app_control, "tag", &query_tag);
  EXPECT_EQ(r, APP_CONTROL_ERROR_NONE);
  EXPECT_STREQ(query_tag, "networking");
}

/*
 * @testcase app_control_set_uri_by_handle_p
 * @description Sets uri data by uri handle.
 * @apicovered app_control_set_uri_by_handle
 */
TEST_F(AppControlTest, app_control_set_uri_by_handle_p) {
  app_control_h app_control;
  char *uri;
  int r = app_control_create(&app_control);
  EXPECT_EQ(r, APP_CONTROL_URI_ERROR_NONE);

  r = app_control_set_uri_by_handle(app_control, uri_);
  EXPECT_EQ(r, APP_CONTROL_URI_ERROR_NONE);

  r = app_control_get_uri(app_control, &uri);
  EXPECT_EQ(r, APP_CONTROL_ERROR_NONE);
  EXPECT_STREQ(uri, EXAM_URI1);
}
